/*
 * Project:		 	3-PHASE AC DRIVE
 *
 * Microcontroller: Motorola MC68HC908MR32 
 *
 * Module:			CONTROL.C
 * Description:		This module is responsible for calculation of control
 *					algorithm of the drive. It is called by MAIN.C s/w
 *					timer and uses subroutines in the file RAMP.C.  
 *
 * Compiler:		Hiware08 
 *
 * Author:			Radim VISINKA
 *
 * ====================================================================
 */


/* DEFINITION_START */
#include <mr32io.h>
#include "CONST.H"					/* file contains global constants
							   and definitions */

/* Global Variables (External) - 8 bit */
extern unsigned char Mcs_status;	/* Status register of MCS */
extern unsigned char Fault_flags;	/* 8 bit fault flag register */
extern unsigned char Dcb_voltage;	/* AD value of DC Bus voltage */
extern unsigned char Amplitude;		/* 0 to 255 gives 0 to 100% modulation*/



/* Global Variables (External) - 16 bit */	
extern signed int Table_inc;		/* increment value for wave pointer update*/		
extern signed int V_out;		/* actual ramp speed */
extern signed int V_command;		/* command speed */
extern signed int V_com_actual;		/* actual speed command */
extern unsigned int Boost_slope;	/* Boost slope pre-calculation */
extern signed int Accel;
extern signed int Decel;


/* Local Variables - 8 bit */
unsigned char temp_y;				/* temp. 8 bit variable */
unsigned char temp_z;				/* temp. 8 bit variable */
unsigned int t_amplitude = 0;  

unsigned char flag1 = 0;			/* 8 bit flag register No.1 */ 
#define ramping  	0x01			/* flag for ramping in flag1 register*/

/* Local Variables - 16 bit */
unsigned int Temp_var_16;			/* temporary 2-bit variable */
signed int V_out_abs;				/* temp. ABS (V_pi) */
signed int Dv;						/* temporary ramp variable */
unsigned int Amplitude_16;			/* temporary 16-bit amplitude */

/* Local Variables - 32 bit */			  
unsigned long Temp_var_32;			/* temp. 32-bit variable */


		  
/* DEFINITION_END */


		/*   SUBROUTINE  ACCEL_RAMP   */

void accel_ramp (void)
	{
					
/* determine whether the ramping flag should be set */

	  if (V_command != V_com_actual)
	  {
		flag1 |= ramping;	/* ramping in progress */
		Dv = Decel;
		Mcs_status &= (~(MCS_ACCEL));

		if (!((V_command ^ V_com_actual)&(0x8000)))	
		{					/* if SIGN(V_command) = SIGN(V_com_actual) */  
											
		  if (abs(V_com_actual) < abs(V_command)) 
		  {					/* if ABS(V_com_actual) < ABS(V_command) */
			Dv = Accel;		
			Mcs_status |= MCS_ACCEL;
		  }
		}  					/* now, appropriate dv is loaded */
		if (V_command > V_com_actual)
		{
		  V_com_actual += Dv;
		  if (V_com_actual >= V_command)
		  {					/* if final speed > command
								   finish ramping */
			V_com_actual = V_command;
			flag1 &= ~(ramping); 
		  }
		}
		else 
		{
		  V_com_actual -= Dv;
		  if (V_com_actual <= V_command)
		  {			 		/* if final speed < command
							   	   finish ramping */
			V_com_actual = V_command;
			flag1 &= ~(ramping);
		  }
		}
	  }
	}							

			/* SUBROUTINE VHZ_RAMP */

void vhz_ramp (void)
	{
  		/* Calculate wave increment Table_inc for rolling thru wavetable */
					
		Table_inc = V_out >> 4;	/* divide by 16 to get propper wave increment
									in 8.8 format for PWM reload=PWM/4 */	

		/* Calculate V/Hz ramp */
			
		V_out_abs = abs(V_out);

		if (V_out_abs < FREQ_BOOST)
		{					/* if ABS(V_out) < FREQ_BOOST */

		Temp_var_32 = (long)Boost_slope * (long)V_out_abs;
		Temp_var_16 = Temp_var_32 / FREQ_BOOST;
		Amplitude_16 = Temp_var_16 + (VOLTS_BOOST * 0x028f);
		Amplitude = Amplitude_16>>8; 	/* 16 to 8 bit */
		  	 
		}
		else
		{
		  if (V_out_abs < FREQ_BASE)
		  {				/* if FREQ_BOOST < ABS(V_out) < FREQ_BASE */
			Amplitude = V_out_abs/(FREQ_BASE>>8);			 			
		  }  
		  else				/* if ABS(V_out) > BASE_FREQ*/
		  {
		    Amplitude = 0xff;
		  }
		}   
	}


void control (void)  
  {
		/* IF disable state, leave the routine */

   if (Mcs_status & MCS_STAND_BY)
   {   
	 V_com_actual = 0;
   }
   else	  /* IF enable state, performe the calculations */
   {	 

		/* Overvoltage Protection During Deceleration */
	
	if ((~(Mcs_status)) & MCS_ACCEL)	/* if deceleration in process */	
	{
	  if (Dcb_voltage > DEC_HIGH_DCB_V)
	  {
		Fault_flags |= OV_DEC_FLAG;
	  }
	  else	/* DCB voltage < high limit */
	  {
		if (Fault_flags & OV_DEC_FLAG)	/* if overvoltage in process */
		{
		  if (Dcb_voltage < DEC_LOW_DCB_V)	  /* DCB voltage < low limit */
		  {
			Fault_flags &= (~(OV_DEC_FLAG));
		  }
		}
	  }
    	}
	
	/* BRAKE CONTROL */

	if (Fault_flags & OV_DEC_FLAG)	/* if deceleration overvoltage */
	{
		PORTE &= ~BRAKE_SWITCH;	/* switch-on brake switch (negative logic) */	
	}
	
	else				/* if no overvoltage */
	{
		PORTE |= BRAKE_SWITCH;	/* switch-off brake switch (negative logic)  */
	}


	accel_ramp ();				/* Acceleration Ramp Calculation */

	   	
	V_out = V_com_actual;  	/* Output frequency = actual frequency command */  			
	
		  	
  	/* Call subroutine ** vhz_ramp ** to calculate the voltage amplitude
  	   according to V/Hz characteristic of the motor  */
  	     	  
 	vhz_ramp();
   }	    	  	 
  }

