/*
 * Project:			3-PHASE AC DRIVE
 *
 * Microcontroller: Motorola MC68HC908MR32 
 *
 * Module:			MAIN.C
 * Description:		File contains initialisation of the system and
 *					software timer.  
 *
 * Compiler:		Hiware08 Compiler
 *
 * Author:			Radim VISINKA
 *
 * ====================================================================
 */




/* DEFINITION_START */

/* Include Header Files */
#include "MR32IO.H"
#include "RAM.H"			/* file contains RAM global variables */
#include "CONST.H"			/* file contains global constants and
							   definitions */

/* Global Variables (External) - 8 bit */
extern unsigned char Amplitude;			/* 0 to 255 gives 0 to 100% modulation*/
extern unsigned char Pwm_count; 		/* counter for PWM interrupt */
extern unsigned char Fault_flags;		/* 8 bit fault flag register */


/* Global Variables (External) - 16 bit */
extern signed int V_command;			/* command speed */
extern signed int V_com_actual;			/* actual ramp speed */
extern signed int Accel;			/* acceleration constant */
extern signed int Decel;			/* deceleration constant */
extern unsigned int Boost_slope;		/* Boost slope pre-calculation */


/* Local Variables - 8 bit */
unsigned char count1;			/* count_down during initialisation */


/* Local Variables - 16 bit */
unsigned int count;			/* count_down during initialisation */
unsigned int Time_ptr_1;		/* time constant 1 for timeout loop SPEED */
unsigned int Time_ptr_2;		/* time constant 2 for timeout loop PI */
unsigned int Temp_tacnt;		/* temporary timer A counter register */

/* Local Variables - 32 bit */
unsigned long temp32;				/* temporary register */


/* External Modules */		
extern void PWMcalc (void);		/* PWM calculation routine */
extern void speed (void);		/* Speed routine */
extern void fault (void);		/* Over Current Fault routine */
extern void control (void);		/* Over Current Fault routine */

/* DEFINITION_END */




void main(void)
	{

			/* setup PLL clock FOR 8 MHz*/

	PBWC = 0x80;			/* set Auto Bandwidth Control */
  	PCTL &= ~(0x20);		/* turn PLL off to be able to program multiplier */
  	PPG=0x87;			/* multiply X-tal frequency by 8 => X-tal=4MHz *8 = 32MHz
			   			bus frequency 32MHz/4 = 8MHz, VCO range = 7 */
 	PCTL |= 0x20;			/* turn on the PLL */
	while (~PBWC & 0x40);  		/* wait for PLL lock */	
    PCTL |= 0x10;    		/* use PLL clock */	



			/* setup PWM module */

	CONFIG = 0x60;			/* 0x00: pos. center PWM mode; cop and LVI enabled */
					/* 0x60: neg. center PWM mode; cop and LVI enabled */
	PMOD = PWM_MODULUS;   		/* set up PWM modulus => PWM frequency */
					/* for 8MHz Bus Frequency 
					    PWM_MODULUS = 0x00fa gives 16kHz PWM */
	DEADTM=15;			/* 2microsecond deadtime = 16 */
	DISMAP=0xff;			/* when PWM disabled, disable PWM1-6 */
	PCTL2 = 0x80;			/* interrupt every 4th. pwm loads */
 	PCTL1 |= 0xc0;			/* disable MCPWM */
	PWMOUT = 0x00;			/* output port control is PWM */
	PCTL1 |= 0x02;  		/* set LDOK bit */ 
	FCR = 0x88;			/* Flt2 and Flt4 enabled in manual mode */

	PVAL1 = PWM_MODULUS/2;		/* set phase A pwm to 50% */
	PVAL3 = PWM_MODULUS/2;		/* set phase B pwm to 50% */
	PVAL5 = PWM_MODULUS/2;		/* set phase C pwm to 50% */

	
			/* setup PORTs input/output */


	DDRA=0x3f;				/* PORT A: 67 inputs, 012345 output */
							/* 0011 1111 */
	PORTA=0x00;    

	PORTA |= YELLOW_LED;		/* PROGRAM_RUNNING_LED ON*/
	PORTA &= ~(GREEN_LED);		/* STATUS_LED OFF*/

	DDRB=0x00;				/* PORT B: A/D */

	DDRC=0x70;				/* PORT C: bits 0-6:inputs */
							/* 000 0000 */

	DDRE=0x01;				/* bit 1-7:input, 0 output */
							/* 0000 0001*/
	PORTE |= BRAKE_SWITCH;		/* BRAKE_SWITCH OFF (negative logic) */

	DDRF=0x00;				/* not used */



			/* setup Timer A for IC (on TCH3A) & for software timer reference */
	
	TASC |= 0x20;			/* stop TIMA */
	TASC &= 0xe0;			/* reset TIMA */
	TASC=TBSC | 0x06;		/* sets prescaller for Clock/64 */
  	TASC3 = 0x44;			/* 0100 0100
					   TCHA3 interrupt enabled
					   selected TCHB0: IC on rising edge
					   x100 01xx */
	 	
			/* setup ADC */
	
	ADCLK = 0x70;			/* ADC clock = internal bus clock / 8 
					   ADC mode: 8-bit truncation */


			/* Initialise Variables */


	Accel = (ACC * CONTROL_CONST)>>8;	/* acceleration */
	Decel = (DEC * CONTROL_CONST)>>8;	/* deceleration */

				/* Initialise boost of V/Hz ramp */
	temp32 = ((long)FREQ_BOOST*(long)0xffff);
	Boost_slope = temp32/FREQ_BASE;

				/* scale the VOLTS_MIN const. and substract */
	Boost_slope = Boost_slope - (VOLTS_BOOST * 0x028f);



	Mcs_status |= MCS_STAND_BY;				/* Stand_by state */


/*****************************************************************/

	
			/* wait */

	count1 = 0x04;
	while (count1 != 0)
	{
	  count1--;
	  count = 0xffff;
	  while (count != 0)
	  {
	    COPCTL = 0x00;	
	    count--;
	  }   
	}

/*****************************************************************/ 
			/* START */
/*****************************************************************/

	PCTL1 |= 0x20; 		/* enables pwm interrupts */		
   	PCTL1 |= 0x01; 		/* enables PWM */
	FTAC |= 0x04;		/* clears FFLAG2 - fault2 flag bit */
	FTAC |= 0x40;		/* clears FFLAG4 - fault2 flag bit */
   
	TASC=TASC & 0xdf;	/* starts timer A - clear stop bit */
    	TASC3 &= 0x7f; 		/* clears CHANNEL0 Flag Bit */

	ADSCR = 0x00;			/* starts AD conversion - Pot_voltage */
					/* ADC channel 0 */
 	Time_ptr_1 = TACNT;		/* initialise software timer pointer */
	Time_ptr_2 = TACNT;
    
   	cli(); 				/* enables interrupt */
 	
 		
endless_loop:

	Temp_tacnt = TACNT;		/* Software Time No. 1 */	
	if ((Temp_tacnt >= Time_ptr_1) && (!((Temp_tacnt > (0xFFFF - READ_CONST)) && (Time_ptr_1 < READ_CONST))))
	{
	  	Time_ptr_1 += READ_CONST;	  
		speed();
	}

	Temp_tacnt = TACNT;		/* Software Time No. 2 */
	if ((Temp_tacnt >= Time_ptr_2) && (!((Temp_tacnt > (0xFFFF - CONTROL_CONST)) && (Time_ptr_2 <= CONTROL_CONST))))
	{
		Time_ptr_2 += CONTROL_CONST;
		control();
	}
	goto endless_loop;
  }	


	
