/*
 * Project:			3-PHASE AC DRIVE
 *
 * Microcontroller: Motorola MC68HC908MR32 
 *
 * Module:			SPEED.C
 * Description:		This routine calculates required speed V_command	
 *					It is called by software timer in MAIN.C
 *
 * Compiler:		Hiware08 Compiler
 *
 * Author:			Radim VISINKA
 *
 * ====================================================================
 */


/* DEFINITION_START */


/* Include Header Files */
#include "MR32IO.H"
#include "CONST.H"			/* file contains global constants and
							   definitions */


/* Global Variables (External) - 8 bit */
extern unsigned char Mcs_status;	/* Status register of MCS */
extern unsigned char Fault_flags;	/* 8 bit fault flag register */
extern unsigned char Amplitude;		/* 0 to 127 gives 0 to 100% modulation*/
extern unsigned char Dcb_voltage;	/* AD value of DC Bus voltage */


/* Global Variables (External) - 16 bit */
extern signed int V_command;		/* command speed */
extern signed int V_com_actual;		/* actual ramp speed */
extern signed int Table_inc;	   	/* increment value for wave pointer update*/


/* Local Variables - 8 bit */
unsigned char Speed_ptr = 0;	/* pointer for speed/time table */
unsigned char Start_cnt = 0;	/* counter for switch-on time of START button*/
unsigned char Ad_pointer = 0;	/* AD conversion pointer*/
unsigned char Pot_voltage = 0;	/* AD value of potentiometer voltage */
unsigned char Flags = 0; 	/* 8 bit flag register for speed routine */
#define SWITCH_INI_FL 	0x10  	/* start/stop initialisation flag */

unsigned char Flt_rec_led_cnt = 0; 	/* flash constant of fault LED */
 
/* Local Variables - 16 bit */
unsigned int Gfr_counter = 0;	/* general fault recovery counter */
			  

/* DEFINITION_END */



void speed (void)
  {
		
	/* INPUT SCAN STATE */

	if (Ad_pointer == 0x00)
	{
	  Pot_voltage = ADRL;	/* read AD result 8-bit */
	  Ad_pointer = 0x01;	/* set-up next conversion */
	  ADSCR = 0x01;			/* starts next conversion - Dcb_voltage */
							/* ADC channel 1 */
	}
	
	
	else   /* if (Ad_pointer == 0x01) */
	{
	  Dcb_voltage = ADRL;	/* read AD result - 8-bit */
	  Ad_pointer = 0x00;	/* set-up next conversion */
	  ADSCR = 0x00;			/* starts next conversion - Pot_voltage */
							/* ADC channel 0 */
	}


	/*    S P E E D   C O M M A N D   C A L C U L A T I O N     */
	  				  
	  if (Mcs_status & START_STOP_FLG)	/* if run demo command accepted */
	  {
		if (PORTA & 0x20)			/* if start switch = start */
		{
		  if (Pot_voltage > 0xf0)
			{
			  Pot_voltage = 0xf0;
			}
		  if (PORTA & 0x10)			  /* if forw/rew switch = forw. */
		  {
		    V_command = Pot_voltage	<< 7;
		  }
		  else							  /* if forw/rew switch = reverse */
		  {
			V_command = - (Pot_voltage << 7);
		  }	
		}
		else  							/* if start switch = stop */
		{
		  V_command = 0x00;	 
  		  Mcs_status &= ~(START_STOP_FLG);	  /* stop */		  
		}
	  }	 

	  else							/* if the motor is stopped */
	  {
	  	if (PORTA & 0x20)			/* if start switch = start */
	    {
	      if (Flags & SWITCH_INI_FL)   	/* if previous switch was = stop */
		  {
			if(Start_cnt == 5)			/* if start_cnt = 5 */
			{
			  Start_cnt = 0;
			  Mcs_status |= START_STOP_FLG;  	/* START flag = 1 */
			}
			else 						/* if start_cnt < 5 */
			{
			Start_cnt += 1;			  	/* increase start_cnt */
			}
		  }
	    }
	    else						 	/* if start swich = stop */
	    {
	      Flags |= SWITCH_INI_FL;  		/* SWITCH_INI_FL = 1 */
	      Start_cnt = 0; 
	    } 
	  }
	

	/* FAULT DETECTION STATE */

	if (Dcb_voltage > DCB_VOLT_LIMIT)	/* checks overvoltage */
	{
	  Fault_flags |= OVER_VOLTAGE;
	}

	/* there can be placed a definition of other fault states */

	if (Fault_flags)	   /* in case of fault */
	{
	
	    Fault_flags = 0;		/* fault flag is cleared*/
		  					
	    Gfr_counter = REC_TIME; 	/*  fault recovery time is loaded */
	 

	  PCTL1 |= 0xc0;				   /* disable MCPWM */
	  FTAC |= 0x04;					/* clears FFLAG2 - fault2 flag bit */

	  Mcs_status |= GF_FLAG;		 /* general fault flag is set*/
	  Mcs_status &= (~MCS_STOP);	
	  Mcs_status &= (~MCS_RUN);	
	  Mcs_status &= (~(GFR_FLAG)); 	 /* gen. fault recovery flag is cleared */
	  Mcs_status &= (~(START_STOP_FLG));	 /* Start/Stop flag is cleared */
	  Flags &= (~(SWITCH_INI_FL));
	  V_com_actual = 0;
	  V_command = 0;						
	}
	else
	{
	  if (Mcs_status & GF_FLAG)		/* if GF flag was set and 
	  										no faults are pending */
	  {
	    Mcs_status |= GFR_FLAG;		/* general fault recovery flag is set*/	
	    Mcs_status &= (~(GF_FLAG)); /* gen. fault recovery flag is cleared */		    	
	  }
	}
 
						
	/* RUN ENABLE STATE */
	
	if ((~(Mcs_status)) & GFR_FLAG)	 /* if no fault occured */
	{
	 if ((~(Mcs_status)) & GF_FLAG)
	 {
	  if (Mcs_status & START_STOP_FLG)
	  {
		PCTL1 &= 0x3f;						/* enable MCPWM */
		if ((V_command == 0) && (V_com_actual == 0))
		{
		  Mcs_status &= (~MCS_RUN);				/* stop state */
		  Mcs_status &= (~MCS_STAND_BY);
		  Mcs_status |= MCS_STOP;		
		}
		else
		{
		  Mcs_status &= (~MCS_STOP);			/* run state */
		  Mcs_status &= (~MCS_STAND_BY);
		  Mcs_status |= MCS_RUN;	
		}
	  }
	  else
	  {
	  	if (V_com_actual == 0)		 
		{
		  PCTL1 |= 0xc0;				   		/* disable MCPWM */
		  Mcs_status &= (~MCS_STOP);			/* stand_by state */
		  Mcs_status &= (~MCS_RUN);
		  Mcs_status |= MCS_STAND_BY;
		}
	  }
	 }
	}
	else					/* if fault recovery required */
	{
	  Gfr_counter--;
	  if (Gfr_counter == 0)
	  {
	   Mcs_status &= (~(GFR_FLAG));
	   Mcs_status |= MCS_STAND_BY;
	  } 	  			
	}


	/* LED DRIVER STATE */

	if (Mcs_status & GF_FLAG)	/* if general fault */
	{
		Flt_rec_led_cnt = FLT_REC_LED_FLASH;
		PORTA &= ~(GREEN_LED);	/* STATUS_LED OFF */
	} 
	if (Mcs_status & GFR_FLAG)  	/* if fault status */
	{	

		Flt_rec_led_cnt--;
		if (Flt_rec_led_cnt == 0)
		{
		  Flt_rec_led_cnt = FLT_REC_LED_FLASH;
		  if (PORTA & GREEN_LED)
		  {
		    PORTA &= ~(GREEN_LED);	/* STATUS_LED OFF */
		  }	
		  else
		  {
		  	PORTA |= GREEN_LED;	/* STATUS_LED ON */
		  }
		}
	}
	  else
	  {
		if (Mcs_status & MCS_STAND_BY)	 /* if STAND_BY status */
		{
		  PORTA &= ~(GREEN_LED);	/* STATUS_LED OFF*/
		}
		else			  /* run or stop status */
		{		  
		  PORTA |= GREEN_LED;	/* STATUS_LED ON */	
		}
	  }
	
  }							
